-- bucher.lua
-- Created on:  2020-10-16
-- Author:      Rétro
--
-- API d'abattage d'arbres pour tortures RSI

---
-- \class Bucheron RSI
local RSI_Bucheron = {}
RSI_Bucheron.__index = RSI_Bucheron

rsiT = require("rsiTurtle")
require("macros")

---
-- \brief Constructeur de RSI_Bucheron
function RSI_Bucheron:init()
    local bucheron = {}
    setmetatable(bucheron,RSI_Bucheron)

    ---
    -- \brief Objet parent
    bucheron.base_ = rsiT:init()

    return bucheron
end

---
-- \brief Affiche la position et l'orientation du bucheron
function RSI_Bucheron:print()
    print("Type bucheron.")
    self.base_:print()
end


---
-- \brief Abattre un arbre
-- \return bool Succès
function RSI_Bucheron:chopTree()
    local success
    local countMovedUp = 0

    success = self.base_:dig(ID_LOG)
    if (not success)
    then
        return false
    end

    success = self.base_:forward()

    print("Chopping down")
    while ( self.base_:dig(ID_LOG, "down") ) do self.base_:down() end
    print("Moving up")

    -- Remonter pour couper les bûches vers le haut. Essaie de trouver des
    -- bûches sur quelques blocs de haut avant d'abandonner et redescendre.
    while (countMovedUp < 3 and self.base_:up())
    do
        countMovedUp = countMovedUp + 1
        printf("Moved up %d\n", countMovedUp)
    end

    print("Chopping up")
    while ( self.base_:dig(ID_LOG, "up") ) do self.base_:up() end
    self.base_:ground()

    return success
end


---
-- \brief Couper les arbres tout en avançant le long d'une ligne
-- \param int   distance  Nombre de blocs à parcourir
-- \return bool Succès
function RSI_Bucheron:moveChop(distance)
    --TODO: Implémenter
    local success = true
    local traveled = 0

    self:chopTree()
    self.base_:ground()
    self.base_:climb()

    while (traveled < distance) and turtle.forward()
    do
        self:chopTree()
        self.base_:ground()
        self.base_:climb()
        self:chopTree()

        traveled = traveled + 1
    end

    if self.facing_ == 0 then
        self.base_.pos_["x"] = self.base_.pos_["x"] + traveled
    elseif self.base_.facing_ == 3 then
        self.base_.pos_["z"] = self.base_.pos_["z"] + traveled
    elseif self.base_.facing_ == 6 then
        self.base_.pos_["x"] = self.base_.pos_["x"] - traveled
    elseif self.base_.facing_ == 9 then
        self.base_.pos_["z"] = self.base_.pos_["z"] - traveled
    end

    return (traveled == distance)
end


---
-- \brief Couper les arbres dans la zone
-- \param int lines Nombre de lignes
-- \param int cols  Nombre de colonnes
-- \return bool Succès
function RSI_Bucheron:chopArea(lines, cols)
    --TODO: Implementer
    local success = true

    assert(lines >= 1)
    assert(cols >= 1)

    -- La ligne de la case de départ compte comme 1
    lines = lines - 1

    self.base_:ground()
    self:chopTree()

    for i=1,cols,1
    do
        print("Debut de colonne.")
        success = self:moveChop(lines) -- FIXME: Remplacer moveForward par moveChop
        self.base_:ground()
        self.base_:climb()

        if not success
        then
            return success
        end

        if ( (i%2) == 0 )
        then
            success = self.base_:left(true)
            self.base_:turnLeft()
        else
            success = self.base_:right(true)
            self.base_:turnRight()
        end
        self:chopTree()

    end
    return success
end


return RSI_Bucheron